package polytech.android.e6201;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

public class LevelActivity extends AppCompatActivity {
    private int displayWidth;  // 幅（画面サイズ）
    private int displayHeight;  // 縦（画面サイズ）
    private int dpi;  // dpi(dots/inch)

    // 各Widget
    private TextView levelHorizontalTextView;
    private TextView levelVerticalTextView;
    RelativeLayout layout;

    private float[] accelerometer = new float[3];  // 加速度センサの値
    private float[] magneticField = new float[3];  // 磁界センサの値

    // 傾きセンサーのイベントリスナ
    private final OrientationSensorEventListener sensorEventListener = new OrientationSensorEventListener();

    // 画面に表示する点のImageView
    private ImageView pointImageView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_level);

        dpi = getResources().getDisplayMetrics().densityDpi;
        Log.v("[Target Device's dpi]", Integer.toString(dpi));

        // TextViewの取得
        levelHorizontalTextView = (TextView) findViewById(R.id.levelHorizontalTextView);
        levelVerticalTextView = (TextView) findViewById(R.id.levelVerticalTextView);

        // 測定点のImageViewの設定
        pointImageView = (ImageView) findViewById(R.id.pointImageView);

        // 「戻る」ボタン
        Button backButton = (Button) findViewById(R.id.backButton3);
        backButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        // Layoutの取得
        layout = (RelativeLayout) findViewById(R.id.relativeLayout);

    }

    // Activityからフォーカスが移った直後か失った直後に発生するメソッド
    // 画面のサイズを取得するために使用
    @Override
    public void onWindowFocusChanged(boolean hasFocus) {
        super.onWindowFocusChanged(hasFocus);

        //ビューサイズの取得
        displayWidth = layout.getWidth() - layout.getPaddingLeft() - layout.getPaddingRight();
        displayHeight = layout.getHeight() - layout.getPaddingTop() - layout.getPaddingBottom();
        Log.v("[View size WIDTH ]", Integer.toString(displayWidth));
        Log.v("[View size HEIGHT]", Integer.toString(displayHeight));

        // 軸の表示
        int topPadding = (displayHeight - displayWidth) / 2;  // 軸の下げ幅
        LevelAxisView levelAxisView = new LevelAxisView(this, displayWidth, topPadding);
        layout.addView(levelAxisView);
    }

    @Override
    protected void onResume() {
        super.onResume();

        // SensorManagerを取得
        SensorManager sensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);

        // SensorManagerから加速度センサを取得
        Sensor sensorAccelerometer = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
        Sensor sensorMagneticField = sensorManager.getDefaultSensor(Sensor.TYPE_MAGNETIC_FIELD);

        // SensorManagerにイベントリスナを登録
        sensorManager.registerListener(sensorEventListener, sensorAccelerometer, SensorManager.SENSOR_DELAY_UI);
        sensorManager.registerListener(sensorEventListener, sensorMagneticField, SensorManager.SENSOR_DELAY_UI);
    }

    @Override
    protected void onPause() {
        super.onPause();

        // SensorManagerを取得
        SensorManager sensorManager = (SensorManager) getSystemService(Context.SENSOR_SERVICE);

        // SensorManagerからイベントリスナの登録を解除します。
        sensorManager.unregisterListener(sensorEventListener);
    }

    private class OrientationSensorEventListener implements SensorEventListener {
        private static final float RAD_TO_DEG = 180 / (float) Math.PI;

        private float[] calcOrientation(float[] accelerometer, float[] magneticField) {
            // 回転行列の宣言
            float[] inR = new float[16];
            float[] outR = new float[16];

            // 回転行列に基づいてデバイスの向きを計算
            SensorManager.getRotationMatrix(inR, null, accelerometer, magneticField);

            // 回転行列inRと向きから回転行列outRを作成【Android端末の水平状態を基準とした場合】
            SensorManager.remapCoordinateSystem(inR, SensorManager.AXIS_X, SensorManager.AXIS_Y, outR);
            // 回転行列inRと向きから回転行列outRを作成【Android端末の垂直状態を基準とした場合】
            //SensorManager.remapCoordinateSystem(inR, SensorManager.AXIS_X, SensorManager.AXIS_Z, outR);

            // 傾きを取得
            float[] orientation = new float[3];
            SensorManager.getOrientation(outR, orientation);

            // 単位を変換
            orientation[0] *= RAD_TO_DEG; // Z軸に対する傾き
            orientation[1] *= RAD_TO_DEG; // X軸に対する傾き
            orientation[2] *= RAD_TO_DEG; // Y軸に対する傾き

            return orientation;
        }

        @Override
        public void onSensorChanged(SensorEvent event) {
            switch (event.sensor.getType()) {
                case Sensor.TYPE_ACCELEROMETER:
                    accelerometer = event.values.clone();
                    break;
                case Sensor.TYPE_MAGNETIC_FIELD:
                    magneticField = event.values.clone();
                    break;
                default:
                    /* NOT REACHED */
            }
            if (accelerometer != null && magneticField != null) {
                float[] orientation = new float[3];
                orientation = calcOrientation(accelerometer, magneticField);  // 傾きの計算
                showLevelTextView(orientation[2], orientation[1]);  // 傾きをTextViewに表示

                // 点の表示座標の計算(画面の中心あたり)。90度を横幅基準の最大にしている。
                int x = displayWidth / 2 + (int) orientation[2] * (displayWidth / 180);
                int y = displayHeight / 2 - (int) orientation[1] * (displayWidth / 180);

                // 円の大きさ(30dp)で表示位置を補正
                int radius = (int) (dpi / 160.0 * 30 / 2); //ピクセル換算した円(30dp)の半径
                x -= radius;
                y -= radius;

                // 点の表示位置の指定
                pointImageView.setTranslationX(x);
                pointImageView.setTranslationY(y);
            }
        }

        private void showLevelTextView(float orientationX, float orientationY) {
            String xString = "水平：" + String.format("%6.1f", orientationY) + "°";
            String yString = "垂直：" + String.format("%6.1f", orientationX) + "°";
            levelHorizontalTextView.setText(xString);
            levelVerticalTextView.setText(yString);
        }

        @Override
        public void onAccuracyChanged(Sensor sensor, int accuracy) {
            /* DO NOTHING */
        }
    }
}
